sg.flybot.pullable.schema

Schema validation and filtering for pullable patterns.

This namespace provides:

  • Schema registry for type inference rules
  • Pattern-schema validation (pre-compilation)
  • Value filtering based on schema structure

Schema rules are functions: (schema) -> {:type :child-schema :valid-keys :key-error} | nil Return nil to indicate the rule doesn't apply to this schema.

Schema validation and filtering for pullable patterns.

This namespace provides:
- Schema registry for type inference rules
- Pattern-schema validation (pre-compilation)
- Value filtering based on schema structure

Schema rules are functions: (schema) -> {:type :child-schema :valid-keys :key-error} | nil
Return nil to indicate the rule doesn't apply to this schema.
raw docstring

get-schema-infoclj/s

(get-schema-info schema)

Get type info from a schema using registered rules. Returns {:type keyword, :child-schema fn-or-nil, :valid-keys set-or-nil, :key-error fn-or-nil}.

Get type info from a schema using registered rules.
Returns {:type keyword, :child-schema fn-or-nil, :valid-keys set-or-nil,
         :key-error fn-or-nil}.
sourceraw docstring

register-schema-rule!clj/s

(register-schema-rule! rule)

Register a rule for schema type inference. Rule: (schema) -> {:type t, :child-schema fn, :valid-keys set, :key-error fn} | nil :key-error (fn [k]) -> why k is invalid, or nil. Return nil if rule doesn't apply. Rules tried in reverse order.

Register a rule for schema type inference.
Rule: (schema) -> {:type t, :child-schema fn, :valid-keys set, :key-error fn} | nil
`:key-error` (fn [k]) -> why `k` is invalid, or nil.
Return nil if rule doesn't apply. Rules tried in reverse order.
sourceraw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries