sg.flybot.pullable.util

Pure utility functions and macros with no domain knowledge.

Pure utility functions and macros with no domain knowledge.
raw docstring

contains-variables?clj/s

(contains-variables? x)

Check if x is or contains any ?-prefixed pattern variable symbol. Walks maps, vectors, lists, and sets.

Check if x is or contains any ?-prefixed pattern variable symbol.
Walks maps, vectors, lists, and sets.
sourceraw docstring

variable?clj/s

(variable? x)

Check if x is a ?-prefixed pattern variable symbol.

Check if x is a ?-prefixed pattern variable symbol.
sourceraw docstring

vars->clj/smacro

(vars-> bindings & body)

Create a fn that destructures vars map. (vars-> [sym child] (mvar sym child)) expands to: (fn [vars] (let [sym (get vars 'sym) child (get vars 'child)] (mvar sym child)))

Create a fn that destructures vars map.
(vars-> [sym child] (mvar sym child))
expands to:
(fn [vars] (let [sym (get vars 'sym) child (get vars 'child)] (mvar sym child)))
sourceraw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries