sg.flybot.pullable

Public API for the pullable pattern matching library.

Public API for the pullable pattern matching library.
raw docstring

sg.flybot.pullable.schema

Schema validation and filtering for pullable patterns.

This namespace provides:

  • Schema registry for type inference rules
  • Pattern-schema validation (pre-compilation)
  • Value filtering based on schema structure

Schema rules are functions: (schema) -> {:type :child-schema :valid-keys} | nil Return nil to indicate the rule doesn't apply to this schema.

Schema validation and filtering for pullable patterns.

This namespace provides:
- Schema registry for type inference rules
- Pattern-schema validation (pre-compilation)
- Value filtering based on schema structure

Schema rules are functions: (schema) -> {:type :child-schema :valid-keys} | nil
Return nil to indicate the rule doesn't apply to this schema.
raw docstring

sg.flybot.pullable.util

Pure utility functions and macros with no domain knowledge.

Pure utility functions and macros with no domain knowledge.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries