Pure utility functions and macros with no domain knowledge.
Pure utility functions and macros with no domain knowledge.
(contains-variables? x)Check if x is or contains any ?-prefixed pattern variable symbol. Walks maps, vectors, lists, and sets.
Check if x is or contains any ?-prefixed pattern variable symbol. Walks maps, vectors, lists, and sets.
(variable? x)Check if x is a ?-prefixed pattern variable symbol.
Check if x is a ?-prefixed pattern variable symbol.
(vars-> bindings & body)Create a fn that destructures vars map. (vars-> [sym child] (mvar sym child)) expands to: (fn [vars] (let [sym (get vars 'sym) child (get vars 'child)] (mvar sym child)))
Create a fn that destructures vars map. (vars-> [sym child] (mvar sym child)) expands to: (fn [vars] (let [sym (get vars 'sym) child (get vars 'child)] (mvar sym child)))
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |